from dwai.llm.bl import AliBlLLM
from dwai.llm.tx import TiOneLLM
from dwai.llm.pangu import HwPanguLLM
from dwai.llm.minimax import MiniMaxLLM
from dwai.llm.azure import AzureChatOpenAI
from dwai.llm.zhipu import ZhiPuLLM
from dwai.llm.wenxin import WenxinLLM
from dwai.llm.fangzhou import FangZhouLLM

DW_LLM_MAP = {
    "ali.qwen": AliBlLLM,
    "hw.pangu": HwPanguLLM,
    "ti.one": TiOneLLM,
    "mini.max": MiniMaxLLM,
    "zhipu.ai": ZhiPuLLM,
    "azure": AzureChatOpenAI,
    "fang.zhou": FangZhouLLM,
    "wenxin": WenxinLLM
}

default_api_key = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"
default_api_base = "https://dwai.shizhuang-inc.com"

class DwlLm(object):
    """获取模型参数"""

    def __call__(self, model_key: str, api_key: str = default_api_key, api_base: str = default_api_base, stream: bool = False, **kwargs):

        if model_key not in DW_LLM_MAP.keys():
            raise RuntimeError("NOT FOUND DW LLM MODEL")

        llm_model = DW_LLM_MAP.get(model_key)
        parameters = kwargs if kwargs else {}

        if model_key == "azure":
            return llm_model(
                deployment_name=kwargs.get("deployment_name"),
                temperature=kwargs.get("temperature"),
                openai_api_version=kwargs.get("openai_api_version"),
                openai_api_key=api_key,
                openai_api_base=api_base,
                openai_api_type=kwargs.get("openai_api_type"),
                openai_organization=kwargs.get("openai_organization"),
                streaming=stream
            )

        return llm_model(
            api_key=api_key,
            api_base=api_base,
            stream=stream,
            parameters=parameters,
        )


def load_dw_llm(model: str, api_key: str = default_api_key, api_base: str = default_api_base, stream: bool = False, **kwargs):

    if model not in DW_LLM_MAP.keys():
        raise RuntimeError("NOT FOUND DW LLM MODEL")

    llm_model = DW_LLM_MAP.get(model)
    parameters = kwargs if kwargs else {}

    if model == "azure":
        return llm_model(
            deployment_name=kwargs.get("deployment_name"),
            temperature=kwargs.get("temperature"),
            openai_api_version=kwargs.get("openai_api_version"),
            openai_api_key=api_key,
            openai_api_base=api_base,
            openai_api_type=kwargs.get("openai_api_type"),
            openai_organization=kwargs.get("openai_organization"),
            streaming=stream
        )

    return llm_model(
        api_key=api_key,
        api_base=api_base,
        stream=stream,
        parameters=parameters,
    )



#!/usr/bin/env python

"""Tests for `dwai` package."""
import unittest
from datetime import datetime

from dwai.pangu.model_api.completions import PanGuCompletions
from dwai.tione.completions import TiOneCompletions
from dwai.bailian.model_api.completions import Completions
from dwai.mini.completions import Completions as MinimaxCompletions
from dwai.pangu.model_api.message import Message

from dwai.zhipuai.model_api.api import ModelAPI
API_KEY = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"
API_BASE = "https://dwai.shizhuang-inc.com"

import dwai

dwai.api_key = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"
dwai.api_base = "https://dwai.shizhuang-inc.com"

class TestDwai(unittest.TestCase):
    """Tests for `dwai` package."""

    def test_bailian_qa(self):
        """Test something."""

        chat = Completions(API_KEY, API_BASE)
        params = {
                "top_p": 0.3,
                "has_thoughts": True
            }

        resp = chat.run_v1("""你是一个智能文档助手，使用以下上下文来回答用户的问题,如果你不知道答案，就说你不知道，不要试图编造答案。
问题: 1+1=?
有用的答案:""", [], False, **params)
        print(resp)

    def test_bailian_qa_stream(self):
        chat = Completions(API_KEY, API_BASE)
        resp = chat.call(
            app_id="1e4ddc3659324ad0b8a0039230f1dba3",
            prompt="你好",
            stream=True)
        for line in resp:
            now = datetime.now()
            print("%s: %s" % (now, line), end="\n", flush=True)

    def test_minimax_qa(self):
        chat = Completions(API_KEY, API_BASE)
    def test_zhipuai_invoke(self):
        model = ModelAPI()
        resp = model.invoke(
            model="chatglm_std",
            prompt=[{"role": "user", "content": "人工智能"}],
            top_p=0.7,
            temperature=0.9,
        )
        print(resp)

    def test_zhipuai_sse_invoke(self):
        model = ModelAPI()
        resp = model.sse_invoke(
            model="chatglm_std",
            prompt=[{"role": "user", "content": "人工智能"}],
            top_p=0.7,
            temperature=0.9,
        )
        print("d.", resp)
        for line in resp:
            now = datetime.now()
            print("%s: %s" % (now, line), end="\n", flush=True)

    def test_zhipuai_async_invoke(self):
        model = ModelAPI
        response = model.async_invoke(
            model="chatglm_std",
            prompt=[{"role": "user", "content": "人工智能"}],
            top_p=0.7,
            temperature=0.9,
        )
        print(response)

    def test_pangu(self):
        """Test something."""
        chat = PanGuCompletions(API_KEY, API_BASE)
        params = dict(max_tokens=600, temperature=0.9)
        messages = [
            Message("system", "请用幼儿园老师的口吻回答问题，注意语气温和亲切，通过提问、引导、赞美等方式，激发学生的思维和想象力。").to_dict(),
            Message("user", "写一首诗").to_dict()
        ]
        resp = chat.run(
            prompt="",
            messages=messages,
            stream=False,
            **params
        )
        print(resp)

    def test_pangu_stream(self):
        """Test something."""
        chat = PanGuCompletions(API_KEY, API_BASE)
        params = dict(max_tokens=600, temperature=0.9,)
        messages = [
            Message("system", "请用幼儿园老师的口吻回答问题，注意语气温和亲切，通过提问、引导、赞美等方式，激发学生的思维和想象力。"),
            Message("user", "写一首诗")
        ]
        resp = chat.run(
            prompt="",
            messages=messages,
            stream=True,
            **params
        )
        # print(resp)
        for line in resp:
            now = datetime.now()
            print("%s: %s" % (now, line), end="\n", flush=True)

    def test_tione(self):
        """Test something."""
        chat = TiOneCompletions(API_KEY, API_BASE)
        params = dict(max_tokens=600, top_p=0.3, )
        resp = chat.run("2+2=?", [], False, **params)
        print(resp)

    def test_minimax(self):
        """Test something."""
        chat = MinimaxCompletions(API_KEY, API_BASE)
        params = {
            "model": "abab5.5-chat",
            "tokens_to_generate": 1034,
            "temperature": 0.01,
            "top_p": 0.95
        }
        resp = chat.run("2+2=?", [], False, **params)
        print(resp)

    def test_wenxin(self):
        from dwai.unified.dw import UnifiedSDK
        qw_sdk = UnifiedSDK("wenxin", API_KEY, API_BASE)
        params = {
            "top_p": 0.8,
            "temperature": 0.95
        }
        body = qw_sdk("1+1=?", [], False, **params)
        print("wenxin:", body.get("result"))

    def test_unified(self):
        from dwai.unified.dw import UnifiedSDK

        # 所有模型自定义参数都放到各自的params中，会自动传入接口Body
        # 阿里千问
        qw_sdk = UnifiedSDK("ali.qwen", API_KEY, API_BASE)
        params = {
            "top_p": 0.3,
            "has_thoughts": True
        }
        print("ali.qs:", qw_sdk("1+1=?", [], False, **params))

        # 华为pangu
        hw_sdk = UnifiedSDK("hw.pangu", API_KEY, API_BASE)
        params = dict(max_tokens=600, temperature=0.9)
        messages = [
            Message("system",
                    "请用幼儿园老师的口吻回答问题，注意语气温和亲切，通过提问、引导、赞美等方式，激发学生的思维和想象力。").to_dict(),
            Message("user", "写一首诗").to_dict()
        ]
        print("hw.qs:", hw_sdk("1+1=?", messages, False, **params))

        # mini Max
        mini_sdk = UnifiedSDK("minimax", API_KEY, API_BASE)
        params = {
            "model": "abab5.5-chat",
            "tokens_to_generate": 1034,
            "temperature": 0.01,
            "top_p": 0.95
        }
        print("mini.qs:", mini_sdk("1+1=?", [], False, **params))

        # 腾讯
        # params = dict(max_tokens=600, top_p=0.3, )
        # ti_sdk = UnifiedSDK("ti.one", API_KEY, API_BASE)
        # print("ti.sdk:", ti_sdk("2+2=?", [], False, **params))

        # 金山智普
        zhipu_sdk = UnifiedSDK("zhipu", API_KEY, API_BASE)
        params = dict(top_p=0.7, temperature=0.9)
        print("zhipu.qs:", zhipu_sdk("你的参数量大概是多少？", [], False, **params))

        # 火山方舟
        fz_sdk = UnifiedSDK("fang.zhou", API_KEY, API_BASE)
        params = dict(temperature=0.9)
        print("ff.qs:", fz_sdk("1+1=？", [], False, **params))

        # 文心
        wx_sdk = UnifiedSDK("wenxin", API_KEY, API_BASE)
        params = {
            "top_p": 0.8,
            "temperature": 0.95
        }
        print("wx.qs:", wx_sdk("1+1=？", [], False, **params))




from dwai.bailian.model_api.completions import Completions as AliBlCompletions
from dwai.pangu.model_api.completions import PanGuCompletions as HwPgCompletions
from dwai.tione.completions import TiOneCompletions
from dwai.mini.completions import Completions as MiniCompletions
from dwai.core.base_completions import BaseCompletions
from dwai.zhipuai.completions import Completions as ZhipuCompletions
from dwai.fangzhou.model_api.completions import Completions as FzCompletions
from dwai.wenxin.model_api.completions import Completions as WenxinCompletions
from typing import Any, List, Dict, Optional

DW_AI_MAP = {
    "ali.qwen": AliBlCompletions,
    "hw.pangu": HwPgCompletions,
    "ti.one": TiOneCompletions,
    "minimax": MiniCompletions,
    "zhipu": ZhipuCompletions,
    "fang.zhou": FzCompletions,
    "wenxin": WenxinCompletions
}

class UnifiedSDK(object):

    client: Optional[BaseCompletions]

    def __init__(self, cloud: str, api_key: str, api_base: str):
        if cloud not in DW_AI_MAP.keys():
            raise RuntimeError("NOT FOUND DW LLM MODEL")

        self.client = DW_AI_MAP.get(cloud)(api_key, api_base)

    def __call__(self, prompt: str, message: List[Dict[str, Any]], stream: bool, **kwargs: Any):
        return self.client.run(prompt, message, stream, **kwargs)



"""Top-level package for dwai."""
import os

api_key = os.environ.get("DW_API_KEY")
api_base_china = os.environ.get("DW_API_CHINA_BASE")
api_base_singapore = os.environ.get("DW_API_SINGAPORE_BASE")

api_timeout_seconds = 300





import json
from typing import Optional, Iterator
from typing import Any, List, Dict
from dwai.core.base_completions import BaseCompletions

class PanGuCompletions(BaseCompletions):

    def run(self, prompt: str, messages: List[Dict[str, Any]], stream: bool, **kwargs: Any):

        headers = dict()

        data = {
            "messages": messages,
            "prompt": prompt,
            "stream": stream,
            **kwargs
        }

        resp = self.do_http(
            "/pangu/v1/chat/completions",
            headers,
            data,
            stream)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data:[DONE]":
                return None
            if line.startswith(b"data:"):
                line = line[len(b"data:"):]
                return line.decode("utf-8")
            else:
                return None
        return None



from typing import Dict


class BaseModel:
    def from_dict(self, data: Dict = None):
        pass

    def to_dict(self):
        pass


# from dwai.bailian.model_api.models import ChatQaMessage
class ChatQaMessage(BaseModel):
    def __init__(self, user, bot):
        self.user = user
        self.bot = bot

    def from_dict(self, data: Dict = None):
        data = data or dict()
        if data.get("User") is not None:
            self.user = data.get("User")
        if data.get("Bot") is not None:
            self.bot = data.get("Bot")

        return self

    def to_dict(self):
        data = dict()
        if self.user is not None:
            data["User"] = self.user
        if self.bot is not None:
            data["Bot"] = self.bot

        return data



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "jsonpath-ng==1.5.3",
    "twine==4.0.2",
    "tencentcloud-sdk-python==3.0.956"
]

test_requirements = []

setup(
    author="罗涛",
    author_email='luotao@shizhuang-inc.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="云商大模型Python SDK",
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='dwai',
    name='dwai',
    packages=find_packages(include=['dwai', 'dwai.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/tluo-github/dwai',
    version='0.4.6',
    zip_safe=False,
)



from langchain.llms.base import LLM
from langchain.callbacks.manager import CallbackManagerForLLMRun
from langchain.llms.utils import enforce_stop_tokens
from langchain.utils import get_from_dict_or_env
from pydantic.class_validators import root_validator

from typing import Any, Dict, List, Optional
from dwai.wenxin.model_api.completions import Completions

class WenxinLLM(LLM):
    client: Any
    api_key: str
    api_base: str
    history: Optional[List[Dict[str, Any]]] = []
    stream: bool = False
    parameters: Optional[Dict[str, Any]] = {}

    def __call__(self, *args, **kwargs):
        return super().__call__(*args, **kwargs)

    @property
    def _llm_type(self) -> str:
        return "WenXinLLM"

    @root_validator()
    def validate_environment(cls, values: Dict) -> Dict:

        if values["client"] is not None:
            return values

        values["api_key"] = get_from_dict_or_env(
            values,
            "api_key",
            "DW_AI_KEY",
        )

        values["api_base"] = get_from_dict_or_env(
            values,
            "api_base",
            "DW_AI_BASE",
        )

        try:
            values["client"] = Completions(values["api_key"], values["api_base"])
        except AttributeError:
            raise ValueError(
                "`openai` has no `HW pangu SDK` attribute, this is likely "
                "due to an old version of the openai package. Try upgrading it "
                "with `pip3.10 install --no-cache-dir  dwai==0.2.1`."
            )

        return values

    def _call(
            self,
            prompt: str,
            stop: Optional[List[str]] = None,
            run_manager: Optional[CallbackManagerForLLMRun] = None,
            **kwargs: Any,
    ) -> str:

        text = ""
        try:
            parameters = {
                "temperature": 0.95,
                "top_p": 0.8
            }
            if self.parameters:
                parameters.update(self.parameters)
            messages = [] if not self.history else self.history
            response = self.client.run(prompt, messages, self.stream, **parameters)
            text = response.get("result",  "")
        except Exception as e:
            raise RuntimeError(f"Error raised by ali bl service: {e}")

        if stop is not None:
            text = enforce_stop_tokens(text, stop)

        return text

